/* (C) 2022 YiRing, Inc. */
package com.yiring.app.service.car.impl;

import com.yiring.app.domain.car.Car;
import com.yiring.app.domain.car.CarRepository;
import com.yiring.app.param.car.CarParam;
import com.yiring.app.param.car.CarQueryParam;
import com.yiring.app.service.car.CarService;
import com.yiring.app.vo.car.CarVo;
import com.yiring.auth.domain.role.Role;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.IdParam;
import com.yiring.common.param.PageParam;
import com.yiring.common.util.StrUtils;
import com.yiring.common.util.date.DateUtils;
import com.yiring.common.vo.PageVo;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

/**
 * @author tzl
 * 2022/4/11 13:58
 */
@Service
public class CarServiceImpl implements CarService {

    @Resource
    CarRepository carRepository;

    @Override
    public Result<Long> saveCarInfo(CarParam carParam) {
        // 检查车牌号是否已存在
        if (hasCarInfo(carParam.getCarNum())) {
            return Result.no(Status.BAD_REQUEST, "车牌号重复");
        }
        Car car = Car
            .builder()
            .carNum(carParam.getCarNum())
            .carType(carParam.getCarType())
            .driverMobile(carParam.getDriverMobile())
            .driverName(carParam.getDriverName())
            .intervieweeId(carParam.getIntervieweeId())
            .orgId(carParam.getOrgId())
            .reason(carParam.getReason())
            .createTime(LocalDateTime.now())
            .build();
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> updateCarInfo(CarParam carParam, IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car car = optional.get();
        if (!car.getCarNum().equals(carParam.getCarNum())) {
            // 当修改车牌号时才检查重复
            if (hasCarInfo(carParam.getCarNum())) {
                return Result.no(Status.BAD_REQUEST, "车牌号重复");
            }
            //如果标签卡在使用中无法修改
            if (StringUtils.isNotBlank(car.getLabelCardStatus()) && "使用中".equals(car.getLabelCardStatus())) {
                return Result.no(Status.BAD_REQUEST, "标签卡使用中,信息无法修改");
            }
        }

        BeanUtils.copyProperties(carParam, car);
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> cardRec(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Car car = optional.get();
        car.setLabelCardStatus("已收卡");
        car.setCardRecTime(LocalDateTime.now());
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<Long> cardSend(IdParam idParam, String labelCard) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }
        Car car = optional.get();
        car.setLabelCardStatus("使用中");
        car.setCardSendTime(LocalDateTime.now());
        car.setLabelCard(labelCard);
        Car carReuslt = carRepository.saveAndFlush(car);
        return Result.ok(carReuslt.getId());
    }

    @Override
    public Result<String> deleteCarInfo(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        //已经发卡的信息无法删除
        Car entity = optional.get();
        if (entity.getLabelCardStatus().equals("使用中")) {
            return Result.no(Status.BAD_REQUEST, "标签卡使用中,信息无法删除");
        }
        carRepository.delete(entity);
        return Result.ok();
    }

    @Override
    public Result<CarVo> getCarInfo(IdParam idParam) {
        Optional<Car> optional = carRepository.findById(idParam.getId());
        if (optional.isEmpty()) {
            return Result.no(Status.NOT_FOUND);
        }

        Car entity = optional.get();
        CarVo vo = new CarVo();
        BeanUtils.copyProperties(entity, vo, Role.Fields.permissions);
        return Result.ok(vo);
    }

    @Override
    public Result<PageVo<CarVo>> PageCarInfo(CarQueryParam carParam, PageParam param) {
        Page<Car> all = carRepository.findAll(condition(carParam), PageParam.toPageable(param));
        List<CarVo> data = all
            .get()
            .map(car -> {
                CarVo vo = new CarVo();
                BeanUtils.copyProperties(car, vo);
                return vo;
            })
            .collect(Collectors.toList());
        PageVo<CarVo> vo = PageVo.build(data, all.getTotalElements());
        return Result.ok(vo);
    }

    @Override
    public List<Car> export(CarQueryParam carParam) {
        return carRepository.findAll(condition(carParam));
    }

    @Override
    public boolean hasCarInfo(String param) {
        Car entity = Car.builder().carNum(param).build();
        return carRepository.count(Example.of(entity)) > 0;
    }

    @Override
    public Specification<Car> condition(CarQueryParam carParam) {
        return (root, query, criteriaBuilder) -> {
            List<Predicate> list = new ArrayList<>();
            //查询条件
            if (StrUtils.isNotBlank(carParam.getCarNum())) {
                //车牌号码查询
                list.add(criteriaBuilder.like(root.get("carNum").as(String.class), "%" + carParam.getCarNum() + "%"));
            }

            if (StrUtils.isNotBlank(carParam.getCarType())) {
                //车辆类型查询
                list.add(criteriaBuilder.like(root.get("carType").as(String.class), "%" + carParam.getCarType() + "%"));
            }

            if (StrUtils.isNotBlank(carParam.getDriverName())) {
                //司机名称查询
                list.add(
                    criteriaBuilder.like(root.get("driverName").as(String.class), "%" + carParam.getDriverName() + "%")
                );
            }

            if (StrUtils.isNotBlank(carParam.getDriverMobile())) {
                //司机电话号码查询
                list.add(
                    criteriaBuilder.like(
                        root.get("driverMobile").as(String.class),
                        "%" + carParam.getDriverMobile() + "%"
                    )
                );
            }

            if (StrUtils.isNotBlank(carParam.getOrgId())) {
                //所属单位查询
                list.add(criteriaBuilder.equal(root.get("orgId").as(String.class), carParam.getOrgId()));
            }

            if (StrUtils.isNotBlank(carParam.getReason())) {
                //来访原因查询
                list.add(criteriaBuilder.like(root.get("reason").as(String.class), "%" + carParam.getReason() + "%"));
            }

            if (StrUtils.isNotNull(carParam.getCreateTimeStart()) && StrUtils.isNotNull(carParam.getCreateTimeEnd())) {
                //来访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get("createTime").as(Date.class),
                        DateUtils.toDate(carParam.getCreateTimeStart()),
                        DateUtils.toDate(carParam.getCreateTimeEnd())
                    )
                );
            }

            if (StrUtils.isNotBlank(carParam.getLabelCard())) {
                //标签卡号查询
                list.add(
                    criteriaBuilder.like(root.get("labelCard").as(String.class), "%" + carParam.getLabelCard() + "%")
                );
            }

            if (StrUtils.isNotBlank(carParam.getLabelCardStatus())) {
                //标签卡状态查询
                list.add(
                    criteriaBuilder.equal(root.get("labelCardStatus").as(String.class), carParam.getLabelCardStatus())
                );
            }

            if (
                StrUtils.isNotNull(carParam.getCardRecTimeStart()) && StrUtils.isNotNull(carParam.getCardRecTimeEnd())
            ) {
                //来访时间区间段查询
                list.add(
                    criteriaBuilder.between(
                        root.get("cardRecTime").as(Date.class),
                        DateUtils.toDate(carParam.getCardRecTimeStart()),
                        DateUtils.toDate(carParam.getCardRecTimeEnd())
                    )
                );
            }

            Predicate[] predicates = new Predicate[list.size()];
            query.where(list.toArray(predicates));
            return criteriaBuilder.and(list.toArray(predicates));
        };
    }
}
